#############################################################################
# Software Testing Automation Framework (STAF)                              #
# (C) Copyright IBM Corp. 2001                                              #
#                                                                           #
# This software is licensed under the Common Public License (CPL) V1.0.     #
#############################################################################

# This file contains tests for the STAF Tcl support
package require STAF
package require STAFMon
package require STAFLog

if {[STAF::Register "Tcl Test"] != 0} {
    puts "Error registering with STAF, RC: $STAF::RC"
    exit $STAF::RC
}

puts "Using handle $STAF::Handle"

#######################################
# First test some basic functionality #
#######################################

puts "Testing basic functionality"

STAF::Submit local ping ping

if {[string compare $STAF::Result PONG] != 0} {
    puts "Wrong output for ping request"
    exit 1
}

if {[STAF::Submit local var "resolve string {STAF/Config/MachineNickname}"] != 0} {
    puts "Error resolving machine nickname, RC: $STAF::RC, Result: $STAF::Result"
    exit $STAF::RC
}

set self $STAF::Result

###############################################
# Next, lets test the monitor service wrapper #
###############################################

puts "Testing Monitor service functions"

# Log the message

set monitorMessage "Hello World"

if {[STAF::Monitor::Log $monitorMessage] != $STAF::kOk} {
    puts "Error on STAF::Monitor::Log, RC: $STAF::RC, Result: $STAF::Result"
    exit $STAF::RC
} 

# Try to retrieve it

set request "query machine $self handle $STAF::Handle"

if {[STAF::Submit local monitor $request] != $STAF::kOk} {
    puts "Error querying monitor info, RC: $STAF::RC, Result: $STAF::Result"
    exit $STAF::RC
} 

# Make sure we got back the correct message

if {[string first $monitorMessage $STAF::Result] == -1} {
    puts "Wrong output for monitor query request"
    puts "Expected to find '<Timestamp> $monitorMessage'"
    puts "Found '$STAF::Result'"
    exit 1
}

##############################################
# Finally, lets test the log service wrapper #
##############################################

puts "Testing Log service functions"

# Setup logging

STAF::Log::Init TclTest handle "Fatal Error Warning Info"

# Log the message

set logMessage "A log message"

if {[STAF::Log::Log info $logMessage] != $STAF::kOk} {
    puts "Error on STAF::Log::Log, RC: $STAF::RC, Result: $STAF::Result"
    exit $STAF::RC
} 

# Try to retrieve it

set request "query machine $self handle $STAF::Handle logname TclTest"

if {[STAF::Submit local log $request] != $STAF::kOk} {
    puts "Error on STAF::Log::Log, RC: $STAF::RC, Result: $STAF::Result"
    exit $STAF::RC
} 

# Make sure we got back the correct message

if {[string first $logMessage $STAF::Result] == -1} {
    puts "Wrong output for log query request"
    puts "Expected to find '<Timestamp> $logMessage'"
    puts "Found '$STAF::Result'"
    exit 1
}

# Try to retrieve it from monitor

set request "query machine $self handle $STAF::Handle"

if {[STAF::Submit local Monitor $request] != $STAF::kOk} {
    puts "Error on querying monitor info, RC: $STAF::RC, Result: $STAF::Result"
    exit $STAF::RC
} 

# Make sure we got back the correct message from monitor

if {[string first INFO:$logMessage $STAF::Result] == -1} {
    puts "Wrong output for monitor query request"
    puts "Expected to find '<Timestamp> INFO:$logMessage'"
    puts "Found '$STAF::Result'"
    exit 1
}

# Delete the log file

set request "delete machine $self handle $STAF::Handle logname TclTest confirm" 

if {[STAF::Submit local log $request] != 0} {
    puts "Error deleting log file, RC: $STAF::RC, Result: $STAF::Result"
    exit 1
}

# Log the message so that Monitor shouldn't get it

if {[STAF::Log::Log status $logMessage] != $STAF::kOk} {
    puts "Error on STAF::Log::Log, RC: $STAF::RC, Result: $STAF::Result"
    exit $STAF::RC
} 

# Try to retrieve it

set request "query machine $self handle $STAF::Handle logname TclTest"

if {[STAF::Submit local log $request] != $STAF::kOk} {
    puts "Error on STAF::Log::Log, RC: $STAF::RC, Result: $STAF::Result"
    exit $STAF::RC
} 

# Make sure we got back the correct message

if {[string first $logMessage $STAF::Result] == -1} {
    puts "Wrong output for log query request"
    puts "Expected to find '<Timestamp> $logMessage'"
    puts "Found '$STAF::Result'"
    exit 1
}

# Try to retrieve it from monitor

set request "query machine $self handle $STAF::Handle"

if {[STAF::Submit local Monitor $request] != $STAF::kOk} {
    puts "Error on querying monitor info, RC: $STAF::RC, Result: $STAF::Result"
    exit $STAF::RC
} 

# Make sure we got back the correct (old) message from monitor

if {[string first INFO:$logMessage $STAF::Result] == -1} {
    puts "Wrong output for monitor query request"
    puts "Expected to find '<Timestamp> INFO:$logMessage'"
    puts "Found '$STAF::Result'"
    exit 1
}

#############
# Finish up #
#############

if {[STAF::UnRegister] != 0} {
    puts "Error unregistering with STAF, RC: $STAF::RC"
    exit $STAF::RC
}

puts "All tests successful"

exit 0
