/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2002, 2004                                        */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.stax;

import javax.swing.*;
import javax.swing.event.*;
import com.ibm.staf.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.*;
import java.io.*;

public class STAXMonitorProcessExtension implements STAXMonitorExtension,
                                                      MouseListener
{
    JPanel fPanel;
    STAFHandle fHandle;
    JTable fProcessTable;
    STAXMonitorTableModel fProcessTableModel;
    STAXMonitorTableSorter fProcessModelSorter;
    Vector fProcessColumns;
    Vector fProcessVector = new Vector();
    Hashtable fProcessIDs = new Hashtable();
    boolean fContinueProcessMonitor = true;
    STAFProcessMonitor fProcessMonitor;
    Hashtable fProcessStartTimes = new Hashtable();
    String fStaxMachine;
    String fStaxServiceName;
    String fJobNumber;
    MonitorElapsedTime fElapsedTime;
    boolean fContinueElapsedTime = true;
    STAXMonitorFrame fMonitorFrame;
    String fTitle;
    Hashtable fProcessHashtable = new Hashtable();

    public JComponent init(STAXMonitorFrame monitorFrame, boolean newJob,
                           String staxMachineName,
                           String staxServiceName, String jobNumber)
                           throws STAFException
    {
        fMonitorFrame = monitorFrame;
        fStaxMachine = staxMachineName;
        fStaxServiceName = staxServiceName;
        fJobNumber = jobNumber;

        fTitle = "Active Processes";

        fPanel = new JPanel();
        fPanel.setLayout(new BorderLayout());

        try
        {
            fHandle = STAXMonitorUtil.getNewSTAFHandle(
                "STAX/JobMonitor/Extension/ProcessTable");
        }
        catch (STAFException ex)
        {
        }

        fProcessColumns = new Vector();
        fProcessColumns.addElement("Process Name");
        fProcessColumns.addElement("Elapsed Time");
        fProcessColumns.addElement("Status");
        fProcessColumns.addElement("Block");
        fProcessColumns.addElement("Machine:Handle");
        fProcessColumns.addElement("Command");

        fProcessTableModel = new STAXMonitorTableModel(fProcessColumns, 0);

        fProcessModelSorter =
            new STAXMonitorTableSorter(fProcessTableModel, 0);

        fProcessTable = new JTable(fProcessModelSorter);
        fProcessModelSorter.addMouseListenerToHeaderInTable(
            fProcessTable, 3);
        fProcessTable.setRowSelectionAllowed(true);
        fProcessTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        fProcessTable.addMouseListener(this);

        fProcessTable.getColumnModel().getColumn(0).setCellRenderer(
            new STAXMonitorTableCellRenderer(new Color(0, 130, 0)));

        fProcessTable.getColumnModel().getColumn(0).
            setHeaderRenderer(new STAXMonitorTableCellRenderer(Color.black,
                true, new Font("Dialog", Font.BOLD, 12)));

        fProcessTable.getColumnModel().getColumn(1).setCellRenderer(
            new STAXMonitorTableCellRenderer(Color.blue));

        fProcessTable.getColumnModel().getColumn(1).
            setHeaderRenderer(new STAXMonitorTableCellRenderer(Color.black,
                true, new Font("Dialog", Font.BOLD, 12)));

        fProcessTable.getColumnModel().getColumn(2).setCellRenderer(
            new STAXMonitorTableCellRenderer(Color.blue));

        fProcessTable.getColumnModel().getColumn(2).
            setHeaderRenderer(new STAXMonitorTableCellRenderer(Color.black,
                true, new Font("Dialog", Font.BOLD, 12)));

        fProcessTable.getColumnModel().getColumn(3).setCellRenderer(
            new STAXMonitorTableCellRenderer(new Color(0, 130, 0)));

        fProcessTable.getColumnModel().getColumn(3).
            setHeaderRenderer(new STAXMonitorTableCellRenderer(Color.black,
                true, new Font("Dialog", Font.BOLD, 12)));

        fProcessTable.getColumnModel().getColumn(4).setCellRenderer(
            new STAXMonitorTableCellRenderer(new Color(0, 130, 0)));


        fProcessTable.getColumnModel().getColumn(4).
            setHeaderRenderer(new STAXMonitorTableCellRenderer(Color.black,
                true, new Font("Dialog", Font.BOLD, 12)));

        fProcessTable.getColumnModel().getColumn(5).setCellRenderer(
            new STAXMonitorTableCellRenderer(new Color(0, 130, 0)));

        fProcessTable.getColumnModel().getColumn(5).
            setHeaderRenderer(new STAXMonitorTableCellRenderer(Color.black,
                true, new Font("Dialog", Font.BOLD, 12)));

        fPanel.add(BorderLayout.CENTER, new JScrollPane(fProcessTable));

        fProcessTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

        STAXMonitorUtil.sizeColumnsToFitText(fProcessTable);

        if (!newJob)
        {
            // Determine the processes that are currently running in the job

            String request = "LIST JOB " + fJobNumber + " PROCESSES";
            STAFResult listResult = fHandle.submit2(
                fStaxMachine, fStaxServiceName, request);

            if (listResult.rc != 0)
            {
                throw new STAFException(
                    listResult.rc,
                    "Error listing Job PROCESSES\n" + listResult.result);
            }

            STAFMarshallingContext mc =
                STAFMarshallingContext.unmarshall(listResult.result);
            java.util.List processList = (java.util.List)mc.getRootObject();
            Iterator iter = processList.iterator();
            
            while (iter.hasNext())
            {
                Map processMap = (Map)iter.next();
                String location = (String)processMap.get("location");
                String handleNumber = (String)processMap.get("handle");

                request = "QUERY JOB " + fJobNumber + " PROCESS " +
                          location + ":" + handleNumber;

                STAFResult queryResult = fHandle.submit2(
                    fStaxMachine, fStaxServiceName, request);

                if (queryResult.rc != 0)
                {
                    continue;
                }

                mc = STAFMarshallingContext.unmarshall(queryResult.result);
                processMap = (HashMap)mc.getRootObject();

                String block = (String)processMap.get("blockName");
                String processName = (String)processMap.get("processName");
                String command = (String)processMap.get("command");
                
                String commandMode = (String)processMap.get("commandMode");
                if (commandMode == null) commandMode = "";
                
                String commandShell = (String)processMap.get("commandShell");
                if (commandShell == null) commandShell = "";
                
                String processTitle = (String)processMap.get("title");
                if (processTitle == null) processTitle = "";
                
                String parms = (String)processMap.get("parms");
                if (parms == null) parms = "";
                
                String workdir = (String)processMap.get("workdir");
                if (workdir == null) workdir = "";
                
                String processID = location + ":" + handleNumber;
                String machineHandleText = location + ":";
                machineHandleText += handleNumber;

                String startTimestamp = 
                    (String)processMap.get("startTimestamp");
                String startDate = startTimestamp.substring(0, 8);
                String startTime = startTimestamp.substring(9);

                synchronized(fProcessStartTimes)
                {
                    fProcessStartTimes.put(
                        processID,
                        STAXMonitorUtil.getCalendar2(startDate, startTime));
                }
                
                Object rowData[] = new Object[6];
                rowData[0] = processName;
                rowData[1] = "";
                rowData[2] = "";
                rowData[3] = block;
                rowData[4] = machineHandleText;
                rowData[5] = command;

                fProcessTableModel.addRow(rowData);

                Vector procDataVector = new Vector();

                addRow(procDataVector, "Location", location);
                addRow(procDataVector, "Handle", handleNumber);
                addRow(procDataVector, "Command", command);
                addRow(procDataVector, "Command Mode", commandMode);
                addRow(procDataVector, "Command Shell", commandShell);

                synchronized(fProcessHashtable)
                {
                    fProcessHashtable.put(processID, procDataVector);
                }

                STAXMonitorUtil.updateRowHeights(fProcessTable, 1);
                STAXMonitorUtil.sizeColumnsToFitText(fProcessTable);
            }
        }

        fProcessMonitor = new STAFProcessMonitor();
        fProcessMonitor.start();

        fElapsedTime = new MonitorElapsedTime();
        fElapsedTime.start();

        return fProcessTable;
    }

    public String getNotificationEventTypes()
    {
        return "process";
    }

    public String getTitle()
    {
        return fTitle;
    }

    public int getExtensionType()
    {
        return STAXMonitorFrame.EXTENSION_ACTIVE;
    }

    public JComponent getComponent()
    {
        return fProcessTable;
    }

    public void mouseClicked(MouseEvent e)
    {
        int selectedRow = fProcessTable.getSelectedRow();

        if (selectedRow == -1)
            return;

        String processID = (String)
            fProcessTable.getValueAt(selectedRow, 4);

        synchronized(fProcessHashtable)
        {
            fMonitorFrame.updateCurrentSelection((String)fProcessTable.
                getValueAt(selectedRow, 0),
                (Vector)fProcessHashtable.get(processID));
        }
    }

    public void mousePressed(MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}

    public void handleEvent(Map map)
    {
        String block = (String)map.get("block");
        String processName = (String)map.get("name");
        String location = (String)map.get("location");
        String machineHandleText = location + ":";
        String handleNumber = (String)map.get("handle");

        machineHandleText += handleNumber;

        String processID = location + ":" + handleNumber;
        
        String command = (String)map.get("command");
        String commandMode = (String)map.get("mode");
        String commandShell = (String)map.get("shell");

        String status = (String)map.get("status");

        if (status.equals("start"))
        {
            String parms = (String)map.get("parms");

            Object rowData[] = new Object[6];
            rowData[0] = processName;
            rowData[1] = " ";
            rowData[2] = " ";
            rowData[3] = block;
            rowData[4] = machineHandleText;
            rowData[5] = command;

            fProcessTableModel.addRow(rowData);

            STAXMonitorUtil.updateRowHeights(fProcessTable, 1);
            STAXMonitorUtil.sizeColumnsToFitText(fProcessTable);

            Vector procDataVector = new Vector();

            addRow(procDataVector, "Location", location);
            addRow(procDataVector, "Handle", handleNumber);
            addRow(procDataVector, "Command", command);
            addRow(procDataVector, "Command Mode", commandMode);

            if (commandShell != null)
                addRow(procDataVector, "Command Shell", commandShell);
            
            String workload = (String)map.get("workload");
            String processTitle = (String)map.get("title");

            if (workload != null)
                addRow(procDataVector, "Workload", workload);
            
            if (processTitle != null)
                addRow(procDataVector, "Title", processTitle);
            
            if (parms != null)
                addRow(procDataVector, "Parms", parms);
            
            if (map.get("varList") instanceof java.util.List)
            {
                java.util.List varList = (java.util.List)map.get("varList");
            
                for (int i = 0; i < varList.size(); i++)
                {
                    addRow(procDataVector, "Var #" + (i + 1),
                           (String)varList.get(i));
                }
            }

            if (map.get("envList") instanceof java.util.List)
            {
                java.util.List envList = (java.util.List)map.get("envList");
            
                for (int i = 0; i < envList.size(); i++)
                {
                    addRow(procDataVector, "Env #" + (i + 1),
                           (String)envList.get(i));
                }
            }

            String useProcessVars = (String)map.get("useprocessvars");
            if (useProcessVars != null)
            {
                addRow(procDataVector, "UseProcessVars", "");
            }

            String stopusing = (String)map.get("stopusing");
            if (stopusing != null)
            {
                addRow(procDataVector, "StopUsing", stopusing);
            }

            String sameConsole = (String)map.get("sameconsole");
            if (sameConsole != null)
            {
                addRow(procDataVector, "SameConsole", "");
            }

            String newConsole = (String)map.get("newconsole");
            if (newConsole != null)
            {
                addRow(procDataVector, "NewConsole", "");
            }

            String username = (String)map.get("username");
            if (username != null)
            {
                addRow(procDataVector, "Username", username);
            }

            String password = (String)map.get("password");
            if (password != null)
            {
                addRow(procDataVector, "Password", password);
            }

            String disabledauthiserror =
                (String)map.get("disabledauthiserror");
            if (disabledauthiserror != null)
            {
                addRow(procDataVector, "DisabledAuthIsError", "");
            }

            String ignoredisabledauth =
                (String)map.get("ignoredisabledauth");
            if (ignoredisabledauth != null)
            {
                addRow(procDataVector, "IgnoreDisabledAuth", "");
            }

            String stdin = (String)map.get("stdin");
            if (stdin != null)
            {
                addRow(procDataVector, "Stdin", stdin);
            }

            String stdout = (String)map.get("stdout");
            if (stdout != null)
            {
                addRow(procDataVector, "Stdout", stdout);
            }

            String stdoutappend = (String)map.get("stdoutappend");
            if (stdoutappend != null)
            {
                addRow(procDataVector, "StdoutAppend", stdoutappend);
            }

            String stderrtostdout = (String)map.get("stderrtostdout");
            if (stderrtostdout != null)
            {
                addRow(procDataVector, "StderrToStdout", "");
            }
            else
            {
                String stderr = (String)map.get("stderr");
                if (stderr != null)
                {
                    addRow(procDataVector, "Stderr", stderr);
                }

                String stderrappend = (String)map.get("stderrappend");
                if (stderrappend != null)
                {
                    addRow(procDataVector, "StderrAppend", stderrappend);
                }
            }

            String returnstdout = (String)map.get("returnstdout");
            if (returnstdout != null)
            {
                addRow(procDataVector, "ReturnStdout", "");
            }

            String returnstderr = (String)map.get("returnstderr");
            if (returnstderr != null)
            {
                addRow(procDataVector, "ReturnStderr", "");
            }

            if (map.get("returnFileList") instanceof java.util.List)
            {
                java.util.List returnFileList = (java.util.List)map.get(
                    "returnFileList");
            
                for (int i = 0; i < returnFileList.size(); i++)
                {
                    addRow(procDataVector, "ReturnFile #" + (i + 1),
                           (String)returnFileList.get(i));
                }
            }

            String other = (String)map.get("other");
            if (other != null)
            {
                addRow(procDataVector, "Other", other);
            }

            STAFResult queryResult =
                fHandle.submit2(fStaxMachine, fStaxServiceName,
                                "QUERY JOB " + fJobNumber + " PROCESS " +
                                location + ":" + handleNumber);

            if (queryResult.rc == 0)
            {
                STAFMarshallingContext mc =
                    STAFMarshallingContext.unmarshall(queryResult.result);
                Map processMap = (HashMap)mc.getRootObject();

                String startTimestamp = 
                    (String)processMap.get("startTimestamp");
                String startDate = startTimestamp.substring(0, 8);
                String startTime = startTimestamp.substring(9);

                synchronized(fProcessStartTimes)
                {
                    fProcessStartTimes.put(
                        processID,
                        STAXMonitorUtil.getCalendar2(startDate, startTime));
                }

                addRow(procDataVector, "Started", startDate + "-" + startTime);

                synchronized(fProcessHashtable)
                {
                    fProcessHashtable.put(processID, procDataVector);
                }
            }
        }
        else if (status.equals("stop"))
        {
            Vector dataVector = fProcessTableModel.getDataVector();

            for (int i = 0; i < dataVector.size(); i++)
            {
                Vector currentRow = (Vector)(dataVector.elementAt(i));

                synchronized(fProcessHashtable)
                {
                    fProcessHashtable.remove(machineHandleText);
                }

                synchronized(fProcessStartTimes)
                {
                    fProcessStartTimes.remove(machineHandleText);
                }

                if (((String)currentRow.elementAt(4)).
                    equalsIgnoreCase(machineHandleText))
                {
                    fProcessTableModel.removeRow(i);
                    STAXMonitorUtil.updateRowHeights(fProcessTable, 1);
                    STAXMonitorUtil.sizeColumnsToFitText(fProcessTable);
                    break;
                }
            }
        }
    }

    public void addRow(Vector vector, String name, String value)
    {
        Vector newRow = new Vector(2);
        newRow.add(name);
        newRow.add(value);
        vector.add(newRow);
    }

    public void term()
    {
        boolean fContinueProcessMonitor = false;

        STAXMonitorUtil.freeHandle(fHandle.getHandle());
    }

    class STAFProcessMonitor extends Thread
    {
        public void run()
        {
            final int frequency =  fMonitorFrame.getProcessMonitorInterval();

            if (frequency <= 0)
                return;

            while (fContinueProcessMonitor)
            {
                int waitTime = frequency;
                Runnable runnable;

                synchronized(fProcessTableModel)
                {
                    final Vector dataVector = fProcessTableModel.getDataVector();

                    runnable = new Runnable()
                    {
                        public void run()
                        {
                            for (int i = 0; i < dataVector.size(); i++)
                            {
                                Vector currentRow = (Vector)(dataVector.elementAt(i));

                                String locationAndHandle = (String)currentRow.
                                    elementAt(4);
                                final String location = locationAndHandle.substring(
                                    0, locationAndHandle.lastIndexOf(":"));
                                final String handle = locationAndHandle.substring(
                                    locationAndHandle.lastIndexOf(":") + 1);

                                final int row = i;

                                String message = STAXMonitorUtil.
                                    getMonitorMessage(location, handle);

                                fProcessTableModel.setValueAt(message, row, 2);
                            }

                            STAXMonitorUtil.sizeColumnsToFitText(fProcessTable);
                            fProcessTable.repaint();
                        }
                    };
                }

                try
                {
                    SwingUtilities.invokeAndWait(runnable);
                }
                catch (InterruptedException ex)
                {
                     ex.printStackTrace();
                }
                catch (InvocationTargetException ex)
                {
                     ex.printStackTrace();
                }

                try
                {
                    Thread.sleep(waitTime);
                }
                catch (InterruptedException ex)
                {
                }
            }
        }
    }

    class MonitorElapsedTime extends Thread
    {
        public void run()
        {
            final int waitTime = fMonitorFrame.getElapsedTimeInterval();

            if (waitTime == 0)
                return;

            while (fContinueElapsedTime)
            {
                synchronized(fProcessTableModel)
                {
                    Runnable processRunnable = new Runnable()
                    {
                        public void run()
                        {
                            Vector dataVector =
                                fProcessTableModel.getDataVector();

                            for (int i = 0; i < dataVector.size(); i++)
                            {
                                Vector currentRow =
                                    (Vector)(dataVector.elementAt(i));

                                String locationAndHandle =
                                    (String)currentRow.elementAt(4);

                                final int row = i;

                                synchronized(fProcessStartTimes)
                                {
                                    Calendar processStarted = (Calendar)
                                        fProcessStartTimes.get(
                                        locationAndHandle);

                                    {
                                        fProcessTableModel.setValueAt(
                                            STAXMonitorUtil.getElapsedTime(
                                            processStarted), row, 1);
                                    }
                                }
                            }

                            fProcessTable.repaint();
                        }
                    };

                    try
                    {
                        SwingUtilities.invokeAndWait(processRunnable);
                    }
                    catch (InterruptedException ex)
                    {
                         ex.printStackTrace();
                    }
                    catch (InvocationTargetException ex)
                    {
                         ex.printStackTrace();
                    }
                }

                try
                {
                    Thread.sleep(waitTime);
                }
                catch (InterruptedException ex)
                {
                }
            }
        }
    }

}