/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2002                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.stax;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;

public class STAXSignalHandlerActionFactory implements STAXActionFactory
{
    private static String fDTDInfo =
"\n" +
"<!--================= The Signal Handler Element =================== -->\n" +
"<!--\n" +
"     The signalhandler element defines how to handle a specified signal.\n" +
"     The signal attribute value is evaluated via Python.\n" +
"-->\n" +
"<!ELEMENT signalhandler (%task;)>\n" +
"<!ATTLIST signalhandler\n" +
"          signal     CDATA        #REQUIRED\n" +
">\n";

    public String getDTDInfo()
    {
        return fDTDInfo;
    }

    public String getDTDTaskName()
    {
        return "signalhandler";
    }

    public STAXAction parseAction(STAX staxService, STAXJob job,
                                  org.w3c.dom.Node root) throws STAXException
    {
        String signalName = new String();
        STAXAction signalAction = null;

        NamedNodeMap attrs = root.getAttributes();

        for (int i = 0; i < attrs.getLength(); ++i)
        {
            Node thisAttr = attrs.item(i);

            String errorInfo = "\n  Element: " + root.getNodeName() +
                               "  Attribute: " + thisAttr.getNodeName();

            if (thisAttr.getNodeName().equals("signal"))
            {
                signalName = STAXUtil.parseAndCompileForPython(
                    thisAttr.getNodeValue(), errorInfo);
            }
            else
            {
                throw new STAXInvalidXMLAttributeException(
                          root.getNodeName() + ": " + thisAttr.getNodeName());
            }
        }

        NodeList children = root.getChildNodes();

        for (int i = 0; i < children.getLength(); ++i)
        {
            Node thisChild = children.item(i);

            if (thisChild.getNodeType() == Node.COMMENT_NODE)
            {
                /* Do nothing */
            }
            else if (thisChild.getNodeType() == Node.ELEMENT_NODE)
            {
                if (signalAction != null)
                {
                    throw new STAXInvalidXMLElementCountException(
                              thisChild.getNodeName());
                }

                STAXActionFactory factory = 
                    staxService.getActionFactory(thisChild.getNodeName());

                if (factory == null)
                {
                    throw new STAXInvalidXMLElementException(
                              thisChild.getNodeName());
                }

                signalAction = factory.parseAction(staxService, job,
                                                   thisChild);
            }
            else
            {
                throw new STAXInvalidXMLNodeTypeException(
                          Integer.toString(thisChild.getNodeType()));
            }
        }

        return new STAXSignalHandlerAction(signalName, signalAction);
    }

    public String getName() { return fName; }

    private String fName;
}
