/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

#include "STAF.h"
#include "STAF_iostream.h"
#include "STAFProcess.h"
#include "STAFThread.h"

int main(int argc, char **argv)
{
    if (argc != 2)
    {
        cout << "Usage: " << argv[0] << " <program>" << endl;
        return 1;
    }

    STAFString command(argv[1]);
    STAFProcessStartInfoLevel1 startInfo = { 0 };

    startInfo.command = command.getImpl();

    STAFProcessID_t pid = 0;
    STAFProcessHandle_t procHandle = 0;
    unsigned int osRC = 0;
    STAFRC_t rc = STAFProcessStart(&pid, &procHandle, &startInfo, 1, &osRC);

    if (rc != kSTAFOk)
    {
        cout << "Error starting process, RC: " << rc << ", OS RC: " << osRC
             << endl;
        return 1;
    }

    cout << "Process ID: " << pid << endl
         << "Process handle: " << procHandle << endl << endl;

    cout << "Sleeping for 5 seconds" << endl;

    STAFThreadSleepCurrentThread(5000, 0);

    rc = STAFProcessStop(pid, kSTAFProcessStopWithSigKill, &osRC);

    if (rc != kSTAFOk)
    {
        cout << "Error stoping process, RC: " << rc << ", OS RC: " << osRC
             << endl;
        return 1;
    }

    return 0;
}
